(*
   This file is part of the Anoq SML Basis project. This project
   implements Anoq's proposals for some extensions to the
   Standard ML Basis Library, which will make the
   Basis Library interfaces more consistent.

   Copyright (C) 2003-2007 Anoq of the Sun (alias Johnny Bock Andersen)

   Contact: http://www.HardcoreProcessing.com/contact
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation;
   version 2.1 of the License.

   As a special exception, when making a "work that uses the library",
   you may compile this library directly into that work, without making
   it possible to relink that work with a modified version of this library.
   This is a relaxation of some of the requirements in section 6 of the
   GNU Lesser General Public License version 2.1.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License version 2.1 for more details.

   If you are interested in a warranty or commercial support for this
   software, contact Hardcore Processing for more information:
   http://www.HardcoreProcessing.com/contact

   You should have received a copy of the GNU Lesser General Public
   License version 2.1 (WITHOUT the option of using a later version)
   along with this library; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*)

(* This is supposed to match all imperative polymorphic collection modules
   in the Standard ML Basis library which support modes of traversal.
   E.g. Array2D. It should also match other polymorphic collections,
   like an Array3D module and a string dictionary with alphabetical
   vs. arbitrary traversal. *)
signature IMPERATIVE_COLLECTION_TRAV =
  sig
    include IMPERATIVE_COLLECTION

    type traversal
    
    (* Supposed to be the default and thus most efficient
       traversal order for the collection data structure. *)
    val defaultTraversal   : traversal

    (* Setting and getting the traversal mode (always a persistent setting) *)
    val setTraversal       : traversal -> 'a t -> 'a t
    val getTraversal       : 'a t -> traversal

    (* Construction operations *)
    val tabulateTraversal  : traversal -> dimensions * (index -> 'a) -> 'a t
    val tabulatelTraversal : traversal -> dimensions * (index * 'b -> 'a * 'b) * 'b -> ('a t) * 'b
    val tabulaterTraversal : traversal -> dimensions * (index * 'b -> 'a * 'b) * 'b -> ('a t) * 'b
  end
