(*
   This file is part of the Anoq SML Basis project. This project
   implements Anoq's proposals for some extensions to the
   Standard ML Basis Library, which will make the
   Basis Library interfaces more consistent.

   Copyright (C) 2003-2007 Anoq of the Sun (alias Johnny Bock Andersen)

   Contact: http://www.HardcoreProcessing.com/contact
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation;
   version 2.1 of the License.

   As a special exception, when making a "work that uses the library",
   you may compile this library directly into that work, without making
   it possible to relink that work with a modified version of this library.
   This is a relaxation of some of the requirements in section 6 of the
   GNU Lesser General Public License version 2.1.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License version 2.1 for more details.

   If you are interested in a warranty or commercial support for this
   software, contact Hardcore Processing for more information:
   http://www.HardcoreProcessing.com/contact

   You should have received a copy of the GNU Lesser General Public
   License version 2.1 (WITHOUT the option of using a later version)
   along with this library; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*)

structure AnoqArray2002 =
  struct
    open Array (* Assumed to be the 1997 implementation of Array *)

    (* For compliance with SML Basis Library 2002 *)
    fun foldli f init v =
          Array.foldli f init (v, 0, NONE)

    fun foldri f init v =
          Array.foldri f init (v, 0, NONE)

    fun appi f v =
          Array.appi f (v, 0, NONE)

    fun modifyi f v =
          Array.modifyi f (v, 0, NONE)
  end

structure AnoqVector2002 =
  struct
    open Vector (* Assumed to be the 1997 implementation of Vector *)

    (* For compliance with SML Basis Library 2002 *)
    fun mapi f v =
          Vector.mapi f (v, 0, NONE)

    fun foldli f init v =
          Vector.foldli f init (v, 0, NONE)

    fun foldri f init v =
          Vector.foldri f init (v, 0, NONE)

    fun appi f v =
          Vector.appi f (v, 0, NONE)
  end

structure AnoqWord8Vector2002 =
  struct
    open Word8Vector (* Assumed to be the 1997 implementation of Word8Vector *)

    (* For compliance with SML Basis Library 2002 *)
    fun mapi f v =
          Word8Vector.mapi f (v, 0, NONE)

    fun foldli f init v =
          Word8Vector.foldli f init (v, 0, NONE)

    fun foldri f init v =
          Word8Vector.foldri f init (v, 0, NONE)

    fun appi f v =
          Word8Vector.appi f (v, 0, NONE)

    (* End compliance with SML Basis Library 2002 *)
  end

structure AnoqCharVector2002 =
  struct
    open CharVector (* Assumed to be the 1997 implementation of CharVector *)

    (* For compliance with SML Basis Library 2002 *)
    fun mapi f v =
          CharVector.mapi f (v, 0, NONE)

    fun foldli f init v =
          CharVector.foldli f init (v, 0, NONE)

    fun foldri f init v =
          CharVector.foldri f init (v, 0, NONE)

    fun appi f v =
          CharVector.appi f (v, 0, NONE)

    (* End compliance with SML Basis Library 2002 *)
  end

structure Substring = (* While we're at it, define a 2002 compliant Substring *)
  struct
    open Substring (* Assumed to be the 1997 implementation of Substring *)

    (* For compliance with SML Basis Library 2002 *)
    val full = Substring.all
  end

structure TextIO = (* While we're at it, define a 2002 compliant TextIO *)
  struct
    open TextIO (* Assumed to be the 1997 implementation of TextIO *)

    (* For compliance with SML Basis Library 2002 *)
    val inputLine = SOME o TextIO.inputLine
  end
