(*
   This file is part of the Anoq SML Basis project. This project
   implements Anoq's proposals for some extensions to the
   Standard ML Basis Library, which will make the
   Basis Library interfaces more consistent.

   Copyright (C) 2003-2007 Anoq of the Sun (alias Johnny Bock Andersen)

   Contact: http://www.HardcoreProcessing.com/contact
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation;
   version 2.1 of the License.

   As a special exception, when making a "work that uses the library",
   you may compile this library directly into that work, without making
   it possible to relink that work with a modified version of this library.
   This is a relaxation of some of the requirements in section 6 of the
   GNU Lesser General Public License version 2.1.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License version 2.1 for more details.

   If you are interested in a warranty or commercial support for this
   software, contact Hardcore Processing for more information:
   http://www.HardcoreProcessing.com/contact

   You should have received a copy of the GNU Lesser General Public
   License version 2.1 (WITHOUT the option of using a later version)
   along with this library; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*)

structure TestAnoqReal =
  struct
    
    fun sameReals (r1::rs1, r2::rs2) =
        Real.==(r1, r2) andalso sameReals(rs1, rs2)
      | sameReals (nil, nil) =
        true
      | sameReals (_, _) =
        false

    fun testForlr () =
          let
            val l0 = Real.forl (0.0, 0,  1.0, op::, nil)
            val l1 = Real.forl (0.0, 10, 1.0, op::, nil)
            val l2 = Real.forl (5.0, 7,  1.0, op::, nil)
            val l3 = Real.forl (3.0, 5,  2.0, op::, nil)

            val r0 = Real.forr (0.0, 0,  1.0, op::, nil)
            val r1 = Real.forr (0.0, 10, 1.0, op::, nil)
            val r2 = Real.forr (5.0, 7,  1.0, op::, nil)
            val r3 = Real.forr (3.0, 5,  2.0, op::, nil)

            val ld0 = Real.forl (0.0, 0,  ~1.0, op::, nil)
            val ld1 = Real.forl (0.0, 10, ~1.0, op::, nil)
            val ld2 = Real.forl (5.0, 7,  ~1.0, op::, nil)
            val ld3 = Real.forl (3.0, 5,  ~2.0, op::, nil)

            val rd0 = Real.forr (0.0, 0,  ~1.0, op::, nil)
            val rd1 = Real.forr (0.0, 10, ~1.0, op::, nil)
            val rd2 = Real.forr (5.0, 7,  ~1.0, op::, nil)
            val rd3 = Real.forr (3.0, 5,  ~2.0, op::, nil)


            val la = (Real.forl (0.0, 10,  0.0, op::, nil); false)
                       handle Subscript => true
            val lc = (Real.forl (0.0, ~1,  1.0, op::, nil); false)
                       handle Subscript => true

            val ra = (Real.forr (0.0, 10,  0.0, op::, nil); false)
                       handle Subscript => true
            val rc = (Real.forr (0.0, ~1,  1.0, op::, nil); false)
                       handle Subscript => true
          in
            sameReals (l0,     []) andalso
            sameReals (l1, rev [0.0, 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0]) andalso
            sameReals (l2, rev [5.0, 6.0, 7.0, 8.0, 9.0, 10.0, 11.0]) andalso
            sameReals (l3, rev [3.0, 5.0, 7.0, 9.0, 11.0]) andalso
            sameReals (r0,     []) andalso
            sameReals (r1,     [0.0, 1.0, 2.0, 3.0, 4.0, 5.0, 6.0, 7.0, 8.0, 9.0]) andalso
            sameReals (r2,     [5.0, 6.0, 7.0, 8.0, 9.0, 10.0, 11.0]) andalso
            sameReals (r3,     [3.0, 5.0, 7.0, 9.0, 11.0]) andalso
            sameReals (ld0,     []) andalso
            sameReals (ld1, rev [0.0, ~1.0, ~2.0, ~3.0, ~4.0, ~5.0, ~6.0, ~7.0, ~8.0, ~9.0]) andalso
            sameReals (ld2, rev [5.0, 4.0, 3.0, 2.0, 1.0, 0.0, ~1.0]) andalso
            sameReals (ld3, rev [3.0, 1.0, ~1.0, ~3.0, ~5.0]) andalso
            sameReals (rd0,     []) andalso
            sameReals (rd1,     [0.0, ~1.0, ~2.0, ~3.0, ~4.0, ~5.0, ~6.0, ~7.0, ~8.0, ~9.0]) andalso
            sameReals (rd2,     [5.0, 4.0, 3.0, 2.0, 1.0, 0.0, ~1.0]) andalso
            sameReals (rd3,     [3.0, 1.0, ~1.0, ~3.0, ~5.0]) andalso
            la andalso lc andalso
            ra andalso rc
          end
    
    fun test () =
          testForlr ()
  end
