(*
   This file is part of the Anoq SML Basis project. This project
   implements Anoq's proposals for some extensions to the
   Standard ML Basis Library, which will make the
   Basis Library interfaces more consistent.

   Copyright (C) 2003-2007 Anoq of the Sun (alias Johnny Bock Andersen)

   Contact: http://www.HardcoreProcessing.com/contact
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation;
   version 2.1 of the License.

   As a special exception, when making a "work that uses the library",
   you may compile this library directly into that work, without making
   it possible to relink that work with a modified version of this library.
   This is a relaxation of some of the requirements in section 6 of the
   GNU Lesser General Public License version 2.1.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License version 2.1 for more details.

   If you are interested in a warranty or commercial support for this
   software, contact Hardcore Processing for more information:
   http://www.HardcoreProcessing.com/contact

   You should have received a copy of the GNU Lesser General Public
   License version 2.1 (WITHOUT the option of using a later version)
   along with this library; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*)

functor FTestAnoqString(structure M : MONO_COLLECTION
                                            where type index = int
                                              and type dimensions = int
                                              and type elem = char
                                              and type t = string) =
  struct
    fun notFindExistsN v n =
          (M.exists (fn i => i = n) v = false) andalso
          (case M.find (fn i => i = n) v of
             NONE => true
           | SOME _ => false
          )

    fun findExistsN v n =
          (M.exists (fn i => i = n) v = true) andalso
          (case M.find (fn i => i = n) v of
             NONE => false
           | SOME _ => true
          )

    fun notFindExistsiN v n =
          (M.existsi (fn (_, e) => e = n) v = false) andalso
          (case M.findi (fn (_, e) => e = n) v of
             NONE => true
           | SOME _ => false
          )

    fun findExistsiN v n =
          (M.existsi (fn (_, e) => e = n) v = true) andalso
          (case M.findi (fn (_, e) => e = n) v of
             NONE => false
           | SOME _ => true
          )

    fun testSimple () =
          let
            val e = M.empty ()

            val test1 = M.dimensions e = 0 andalso
                        M.count e = 0 andalso
                        notFindExistsN e (chr 0)

            val e1 = M.insert 0 (chr 20) e
            val test2 = M.dimensions e1 = 1 andalso
                        M.count e1 = 1 andalso
                        M.sub(e1, 0) = (chr 20) andalso
                        notFindExistsN e1 (chr 0)

            val e2 = M.create (3, chr 14)
            val test3 = M.dimensions e2 = 3 andalso
                        M.count e2 = 3 andalso
                        M.sub(e2, 0) = chr 14 andalso
                        M.sub(e2, 1) = chr 14 andalso
                        M.sub(e2, 2) = chr 14 andalso
                        notFindExistsN e2 (chr 0) andalso
                        notFindExistsN e2 (chr 3) andalso
                        findExistsN e2 (chr 14)

          in
            test1 andalso test2 andalso test3
          end


    fun testBasic () =
          let
            val v = M.tabulate(10, fn i => chr i)

            val test1 = notFindExistsN v (chr 20)
            val test2 = findExistsN v (chr 9)

            val v1 = M.insert 5 (chr 20) v
            val test3 = M.sub(v1, 5) = (chr 20) andalso
                        M.sub(v1, 4) = (chr 4)  andalso
                        M.sub(v1, 6) = (chr 5)

            val test4 = findExistsN v1 (chr 20)
            val test5 = findExistsN v1 (chr 9)
            val test6 = notFindExistsN v (chr 20)
            val test7 = findExistsN v (chr 9)

            val v2 = M.delete 6 v1
            val test8 = (M.sub(v2, 5)) = (chr 20) andalso
                        (M.sub(v2, 6)) = (chr 6)  andalso
                        (case M.lookup v2 6 of
                           SOME c =>
                             c = chr 6
                         | _ =>
                             false
                        )

            val v3 = M.substitute 3 (fn _ => chr 15) v2
            val test9 = M.sub(v3, 2) = (chr 2) andalso
                        M.sub(v3, 3) = (chr 15)  andalso
                        M.sub(v3, 4) = (chr 4)

          in
            test1 andalso test2 andalso
            test3 andalso test4 andalso
            test5 andalso test6 andalso
            test7 andalso test8 andalso
            test9
          end
    
    fun testBasici () =
          let
            val v = M.tabulate(10, fn i => chr i)

            val test1 = notFindExistsiN v (chr 20)
            val test2 = findExistsiN v (chr 9)

            val v1 = M.insert 5 (chr 20) v
            val test3 = M.sub(v1, 5) = (chr 20) andalso
                        M.sub(v1, 4) = (chr 4)  andalso
                        M.sub(v1, 6) = (chr 5)

            val test4 = findExistsiN v1 (chr 20)
            val test5 = findExistsiN v1 (chr 9)
            val test6 = notFindExistsiN v (chr 20)
            val test7 = findExistsiN v (chr 9)

            val v2 = M.delete 6 v1
            val test8 = (M.sub(v2, 5)) = (chr 20) andalso
                        (M.sub(v2, 6)) = (chr 6)  andalso
                        (case M.lookup v2 6 of
                           SOME c =>
                             c = chr 6
                         | _ => false
                        )

            val v3 = M.substitute 3 (fn _ => chr 15) v2
            val test9 = M.sub(v3, 2) = (chr 2) andalso
                        M.sub(v3, 3) = (chr 15)  andalso
                        M.sub(v3, 4) = (chr 4)

          in
            test1 andalso test2 andalso
            test3 andalso test4 andalso
            test5 andalso test6 andalso
            test7 andalso test8 andalso
            test9
          end
    
    fun testTabulatelr () =
          let
            fun sum (e, acc) =
                  (chr acc, e + acc)

            val (l1, r1) = M.tabulatel (5, sum, 100)
            val (l2, r2) = M.tabulater (5, sum, 100)

            val (l3, r3) = M.tabulatel (2, sum, 100)
            val (l4, r4) = M.tabulater (2, sum, 100)

            val (l5, r5) = M.tabulatel (1, sum, 100)
            val (l6, r6) = M.tabulater (1, sum, 100)

            val (l7, r7) = M.tabulatel (0, sum, 100)
            val (l8, r8) = M.tabulater (0, sum, 100)
          in
            M.toList l1   = [chr 100, chr 100, chr 101, chr 103, chr 106] andalso
            r1 = 110 andalso
            M.toList l2   = [chr 110, chr 109, chr 107, chr 104, chr 100] andalso
            r2 = 110 andalso
            M.toList l3   = [chr 100, chr 100] andalso
            r3 = 101 andalso
            M.toList l4   = [chr 101, chr 100] andalso
            r4 = 101 andalso
            M.toList l5   = [chr 100] andalso
            r5 = 100 andalso
            M.toList l6   = [chr 100] andalso
            r6 = 100 andalso
            M.toList l7   = [] andalso
            r7 = 100 andalso
            M.toList l8   = [] andalso
            r8 = 100
          end
    
    fun testListReps () =
          (M.toList (M.fromList [chr 1, chr 4, chr 6])) = [chr 1, chr 4, chr 6] andalso
          (M.toList (M.fromList [chr 5, chr 0, chr 1])) = [chr 5, chr 0, chr 1] andalso
          (M.toList (M.fromList [chr 5])) = [chr 5] andalso
          (M.toList (M.fromList [])) = []

    fun testRevAppend () =
          let
            val c5 = M.tabulate(5, fn x => chr x)
            val c2 = M.tabulate(2, fn x => chr x)
            val c1 = M.tabulate(1, fn x => chr x)
            val c0 = M.tabulate(0, fn x => chr x)
            val c1_9 = M.tabulate(1, fn _ => chr 9)

            val r5 = M.rev c5
            val r2 = M.rev c2
            val r1 = M.rev c1
            val r0 = M.rev c0

            val a5 = M.append (c5, c5)
            val a2 = M.append (c2, c2)
            val a1_9 = M.append (c1, c1_9)
            val a0 = M.append (c0, c0)

            val a50 = M.append (c5, c0)
            val a51 = M.append (c5, c1)
            val a05 = M.append (c0, c5)
            val a1r5 = M.append (c1, r5)
          in
            M.toList r5   = [chr 4, chr 3, chr 2, chr 1, chr 0] andalso
            M.toList r2   = [chr 1, chr 0] andalso
            M.toList r1   = [chr 0] andalso
            M.toList r0   = [] andalso
            M.toList a5   = [chr 0, chr 1, chr 2, chr 3, chr 4, chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList a2   = [chr 0, chr 1, chr 0, chr 1] andalso
            M.toList a1_9 = [chr 0, chr 9] andalso
            M.toList a0   = [] andalso
            M.toList a50  = [chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList a05  = [chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList a51  = [chr 0, chr 1, chr 2, chr 3, chr 4, chr 0] andalso
            M.toList a1r5 = [chr 0, chr 4, chr 3, chr 2, chr 1, chr 0]
          end
    
    fun testConcat () =
          let
            val c5 = M.tabulate(5, fn x => chr x)
            val c2 = M.tabulate(2, fn x => chr x)
            val c1 = M.tabulate(1, fn x => chr x)
            val c0 = M.tabulate(0, fn x => chr x)
            val c1_9 = M.tabulate(1, fn _ => chr 9)

            val r5 = M.rev c5
            val r2 = M.rev c2
            val r1 = M.rev c1
            val r0 = M.rev c0

            val a5 = M.concat [c5, c5]
            val a2 = M.concat [c2, c2]
            val a1_9 = M.concat [c1, c1_9]
            val a0 = M.concat [c0, c0]

            val a50 = M.concat [c5, c0]
            val a51 = M.concat [c5, c1]
            val a05 = M.concat [c0, c5]
            val a1r5 = M.concat [c1, r5]

            (* Concatenate 3, 1 and 0 collections *)
            val c5c1r5 = M.concat [c5, c1, r5]
            val cc5 = M.concat [c5]
            val ce = M.concat []
          in
            M.toList r5   = [chr 4, chr 3, chr 2, chr 1, chr 0] andalso
            M.toList r2   = [chr 1, chr 0] andalso
            M.toList r1   = [chr 0] andalso
            M.toList r0   = [] andalso
            M.toList a5   = [chr 0, chr 1, chr 2, chr 3, chr 4, chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList a2   = [chr 0, chr 1, chr 0, chr 1] andalso
            M.toList a1_9 = [chr 0, chr 9] andalso
            M.toList a0   = [] andalso
            M.toList a50  = [chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList a05  = [chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList a51  = [chr 0, chr 1, chr 2, chr 3, chr 4, chr 0] andalso
            M.toList a1r5 = [chr 0, chr 4, chr 3, chr 2, chr 1, chr 0] andalso
            M.toList c5c1r5 = [chr 0, chr 1, chr 2, chr 3, chr 4, chr 0, chr 4, chr 3, chr 2, chr 1, chr 0] andalso
            M.toList cc5  = [chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList ce   = []
          end
          
    fun testApp () =
          let
            fun returnAppElems l =
                  let
                    val acc = ref nil : char list ref

                    fun accElem e =
                          acc := (e::(!acc))
                    
                    val _ = M.app accElem l
                  in
                    List.rev (!acc)
                  end

            val c5 = M.tabulate(5, fn x => chr x)
            val c2 = M.tabulate(2, fn x => chr x)
            val c1 = M.tabulate(1, fn x => chr x)
            val c0 = M.tabulate(0, fn x => chr x)

            val a5 = returnAppElems c5
            val a2 = returnAppElems c2
            val a1 = returnAppElems c1
            val a0 = returnAppElems c0
          in
            a5   = [chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            a2   = [chr 0, chr 1] andalso
            a1   = [chr 0] andalso
            a0   = []
          end

    fun testAppi () =
          let
            fun returnAppElems l =
                  let
                    val acc = ref nil : (int * char) list ref

                    fun accElem e =
                          acc := (e::(!acc))
                    
                    val _ = M.appi accElem l
                  in
                    List.rev (!acc)
                  end

            val c5 = M.tabulate(5, fn x => chr x)
            val c2 = M.tabulate(2, fn x => chr x)
            val c1 = M.tabulate(1, fn x => chr x)
            val c0 = M.tabulate(0, fn x => chr x)

            val a5 = returnAppElems c5
            val a2 = returnAppElems c2
            val a1 = returnAppElems c1
            val a0 = returnAppElems c0
          in
            a5   = [(0, chr 0), (1, chr 1), (2, chr 2), (3, chr 3), (4, chr 4)] andalso
            a2   = [(0, chr 0), (1, chr 1)] andalso
            a1   = [(0, chr 0)] andalso
            a0   = []
          end
    
    fun testMap () =
          let
            val c5 = M.tabulate(5, fn x => chr x)
            val c2 = M.tabulate(2, fn x => chr x)
            val c1 = M.tabulate(1, fn x => chr x)
            val c0 = M.tabulate(0, fn x => chr x)

            val a5 = M.map (fn i => chr (ord i + 1)) c5
            val a2 = M.map (fn i => chr (ord i + 1)) c2
            val a1 = M.map (fn i => chr (ord i + 1)) c1
            val a0 = M.map (fn i => chr (ord i + 1)) c0
          in
            M.toList a5   = [chr 1, chr 2, chr 3, chr 4, chr 5] andalso
            M.toList a2   = [chr 1, chr 2] andalso
            M.toList a1   = [chr 1] andalso
            M.toList a0   = []
          end

    fun testMapi () =
          let
            val c5 = M.tabulate(5, fn x => chr x)
            val c2 = M.tabulate(2, fn x => chr x)
            val c1 = M.tabulate(1, fn x => chr x)
            val c0 = M.tabulate(0, fn x => chr x)

            val a5 = M.mapi (fn (i, _) => chr (i + 1)) c5
            val a2 = M.mapi (fn (i, _) => chr (i + 1)) c2
            val a1 = M.mapi (fn (i, _) => chr (i + 1)) c1
            val a0 = M.mapi (fn (i, _) => chr (i + 1)) c0

            val b5 = M.mapi (fn (_, e) => chr (ord e + 1)) a5
            val b2 = M.mapi (fn (_, e) => chr (ord e + 1)) a2
            val b1 = M.mapi (fn (_, e) => chr (ord e + 1)) a1
            val b0 = M.mapi (fn (_, e) => chr (ord e + 1)) a0

            val i5 = M.mapi (fn (i, e) => if chr i = e then chr 1 else chr 0) c5
            val i2 = M.mapi (fn (i, e) => if chr i = e then chr 1 else chr 0) c2
            val i1 = M.mapi (fn (i, e) => if chr i = e then chr 1 else chr 0) c1
            val i0 = M.mapi (fn (i, e) => if chr i = e then chr 1 else chr 0) c0
          in
            M.toList a5   = [chr 1, chr 2, chr 3, chr 4, chr 5] andalso
            M.toList a2   = [chr 1, chr 2] andalso
            M.toList a1   = [chr 1] andalso
            M.toList a0   = [] andalso
            M.toList b5   = [chr 2, chr 3, chr 4, chr 5, chr 6] andalso
            M.toList b2   = [chr 2, chr 3] andalso
            M.toList b1   = [chr 2] andalso
            M.toList b0   = [] andalso
            M.toList i5   = [chr 1, chr 1, chr 1, chr 1, chr 1] andalso
            M.toList i2   = [chr 1, chr 1] andalso
            M.toList i1   = [chr 1] andalso
            M.toList i0   = []
          end
    
    fun testFold () =
          let
            val c5 = M.tabulate(5, fn x => chr x)
            val c2 = M.tabulate(2, fn x => chr x)
            val c1 = M.tabulate(1, fn x => chr x)
            val c0 = M.tabulate(0, fn x => chr x)

            val l5 = M.foldl op:: nil c5
            val l2 = M.foldl op:: nil c2
            val l1 = M.foldl op:: nil c1
            val l0 = M.foldl op:: nil c0

            val r5 = M.foldr op:: nil c5
            val r2 = M.foldr op:: nil c2
            val r1 = M.foldr op:: nil c1
            val r0 = M.foldr op:: nil c0
          in
            l5   = [chr 4, chr 3, chr 2, chr 1, chr 0] andalso
            l2   = [chr 1, chr 0] andalso
            l1   = [chr 0] andalso
            l0   = [] andalso
            r5   = [chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            r2   = [chr 0, chr 1] andalso
            r1   = [chr 0] andalso
            r0   = []
          end

    fun testFoldi () =
          let
            val c5 = M.tabulate(5, fn x => chr x)
            val c2 = M.tabulate(2, fn x => chr x)
            val c1 = M.tabulate(1, fn x => chr x)
            val c0 = M.tabulate(0, fn x => chr x)

            val l5 = M.foldli (fn (i, e, acc) => e::acc) nil c5
            val l2 = M.foldli (fn (i, e, acc) => e::acc) nil c2
            val l1 = M.foldli (fn (i, e, acc) => e::acc) nil c1
            val l0 = M.foldli (fn (i, e, acc) => e::acc) nil c0

            val r5 = M.foldri (fn (i, e, acc) => e::acc) nil c5
            val r2 = M.foldri (fn (i, e, acc) => e::acc) nil c2
            val r1 = M.foldri (fn (i, e, acc) => e::acc) nil c1
            val r0 = M.foldri (fn (i, e, acc) => e::acc) nil c0

            val li5 = M.foldli (fn (i, e, acc) => acc andalso chr i = e) true c5
            val li2 = M.foldli (fn (i, e, acc) => acc andalso chr i = e) true c2
            val li1 = M.foldli (fn (i, e, acc) => acc andalso chr i = e) true c1
            val li0 = M.foldli (fn (i, e, acc) => acc andalso chr i = e) true c0

            val ri5 = M.foldri (fn (i, e, acc) => acc andalso chr i = e) true c5
            val ri2 = M.foldri (fn (i, e, acc) => acc andalso chr i = e) true c2
            val ri1 = M.foldri (fn (i, e, acc) => acc andalso chr i = e) true c1
            val ri0 = M.foldri (fn (i, e, acc) => acc andalso chr i = e) true c0
          in
            l5   = [chr 4, chr 3, chr 2, chr 1, chr 0] andalso
            l2   = [chr 1, chr 0] andalso
            l1   = [chr 0] andalso
            l0   = [] andalso
            r5   = [chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            r2   = [chr 0, chr 1] andalso
            r1   = [chr 0] andalso
            r0   = [] andalso
            li5 andalso li2 andalso li1 andalso li0 andalso
            ri5 andalso ri2 andalso ri1 andalso ri0
          end

    fun testFoldsep () =
          let
            val c5 = M.tabulate(5, fn x => chr x)
            val c2 = M.tabulate(2, fn x => chr x)
            val c1 = M.tabulate(1, fn x => chr x)
            val c0 = M.tabulate(0, fn x => chr x)

            val l5 = M.foldsepl (fn acc => ", "::acc)
                                (fn (e, acc) => (Int.toString (ord e))::acc)
                                nil c5
            val l2 = M.foldsepl (fn acc => ", "::acc)
                                (fn (e, acc) => (Int.toString (ord e))::acc)
                                nil c2
            val l1 = M.foldsepl (fn acc => ", "::acc)
                                (fn (e, acc) => (Int.toString (ord e))::acc)
                                nil c1
            val l0 = M.foldsepl (fn acc => ", "::acc)
                                (fn (e, acc) => (Int.toString (ord e))::acc)
                                nil c0

            val r5 = M.foldsepr (fn acc => ", "::acc)
                                (fn (e, acc) => (Int.toString (ord e))::acc)
                                nil c5
            val r2 = M.foldsepr (fn acc => ", "::acc)
                                (fn (e, acc) => (Int.toString (ord e))::acc)
                                nil c2
            val r1 = M.foldsepr (fn acc => ", "::acc)
                                (fn (e, acc) => (Int.toString (ord e))::acc)
                                nil c1
            val r0 = M.foldsepr (fn acc => ", "::acc)
                                (fn (e, acc) => (Int.toString (ord e))::acc)
                                nil c0
          in
            String.concat l5   = "4, 3, 2, 1, 0" andalso
            String.concat l2   = "1, 0" andalso
            String.concat l1   = "0" andalso
            String.concat l0   = "" andalso
            String.concat r5   = "0, 1, 2, 3, 4" andalso
            String.concat r2   = "0, 1" andalso
            String.concat r1   = "0" andalso
            String.concat r0   = ""
          end

    fun testFoldsepi () =
          let
            val c5 = M.tabulate(5, fn x => chr x)
            val c2 = M.tabulate(2, fn x => chr x)
            val c1 = M.tabulate(1, fn x => chr x)
            val c0 = M.tabulate(0, fn x => chr x)

            val l5 = M.foldsepli (fn acc => ", "::acc) (fn (i, e, acc) => (Int.toString (ord e))::acc) nil c5
            val l2 = M.foldsepli (fn acc => ", "::acc) (fn (i, e, acc) => (Int.toString (ord e))::acc) nil c2
            val l1 = M.foldsepli (fn acc => ", "::acc) (fn (i, e, acc) => (Int.toString (ord e))::acc) nil c1
            val l0 = M.foldsepli (fn acc => ", "::acc) (fn (i, e, acc) => (Int.toString (ord e))::acc) nil c0

            val r5 = M.foldsepri (fn acc => ", "::acc) (fn (i, e, acc) => (Int.toString (ord e))::acc) nil c5
            val r2 = M.foldsepri (fn acc => ", "::acc) (fn (i, e, acc) => (Int.toString (ord e))::acc) nil c2
            val r1 = M.foldsepri (fn acc => ", "::acc) (fn (i, e, acc) => (Int.toString (ord e))::acc) nil c1
            val r0 = M.foldsepri (fn acc => ", "::acc) (fn (i, e, acc) => (Int.toString (ord e))::acc) nil c0

            val li5 = M.foldsepli (fn b => b) (fn (i, e, acc) => acc andalso chr i = e) true c5
            val li2 = M.foldsepli (fn b => b) (fn (i, e, acc) => acc andalso chr i = e) true c2
            val li1 = M.foldsepli (fn b => b) (fn (i, e, acc) => acc andalso chr i = e) true c1
            val li0 = M.foldsepli (fn b => b) (fn (i, e, acc) => acc andalso chr i = e) true c0

            val ri5 = M.foldsepri (fn b => b) (fn (i, e, acc) => acc andalso chr i = e) true c5
            val ri2 = M.foldsepri (fn b => b) (fn (i, e, acc) => acc andalso chr i = e) true c2
            val ri1 = M.foldsepri (fn b => b) (fn (i, e, acc) => acc andalso chr i = e) true c1
            val ri0 = M.foldsepri (fn b => b) (fn (i, e, acc) => acc andalso chr i = e) true c0
          in
            String.concat l5   = "4, 3, 2, 1, 0" andalso
            String.concat l2   = "1, 0" andalso
            String.concat l1   = "0" andalso
            String.concat l0   = "" andalso
            String.concat r5   = "0, 1, 2, 3, 4" andalso
            String.concat r2   = "0, 1" andalso
            String.concat r1   = "0" andalso
            String.concat r0   = "" andalso
            li5 andalso li2 andalso li1 andalso li0 andalso
            ri5 andalso ri2 andalso ri1 andalso ri0
          end
    
    fun testFilter () =
          let
            val c = M.tabulate (5, fn x => chr x)

            val all  = M.filter (fn _ => true) c
            val none = M.filter (fn _ => false) c
            val even = M.filter (fn i => (ord i) mod 2 = 0) c
            val odd  = M.filter (fn i => (ord i) mod 2 = 1) c
          in
            M.toList all   = [chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList none  = [] andalso
            M.toList even  = [chr 0, chr 2, chr 4] andalso
            M.toList odd   = [chr 1, chr 3]
          end

    fun testFilteri () =
          let
            val c = M.tabulate (5, fn x => chr x)

            val all  = M.filteri (fn _ => true) c
            val none = M.filteri (fn _ => false) c
            val even = M.filteri (fn (i, _) => i mod 2 = 0) c
            val odd  = M.filteri (fn (i, _) => i mod 2 = 1) c
            
            val all' = M.filteri (fn (i, e) => chr i = e) c
          in
            M.toList all   = [chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList all'  = [chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList none  = [] andalso
            M.toList even  = [chr 0, chr 2, chr 4] andalso
            M.toList odd   = [chr 1, chr 3]
          end
    
    fun testSplitAt () =
          let
            val c = M.tabulate (5, fn x => chr x)

            val (l1, r1) = M.splitAt (c, 2)
            val (l2, r2) = M.splitAt (c, 4)
            
            val (l3, r3) = M.splitAt (c, 1)
            val (l4, r4) = M.splitAt (c, 5)
            val (l5, r5) = M.splitAt (c, 0)
            
            val l1'  = M.trimr 3 c
            val l1'' = M.keepl 2 c
            val l2'  = M.trimr 1 c
            val l2'' = M.keepl 4 c
            val r1'  = M.triml 2 c
            val r1'' = M.keepr 3 c
            val r2'  = M.triml 4 c
            val r2'' = M.keepr 1 c
            
            val l3'  = M.trimr 4 c
            val l3'' = M.keepl 1 c
            val l4'  = M.trimr 0 c
            val l4'' = M.keepl 5 c
            val l5'  = M.trimr 5 c
            val l5'' = M.keepl 0 c
            val r3'  = M.triml 1 c
            val r3'' = M.keepr 4 c
            val r4'  = M.triml 5 c
            val r4'' = M.keepr 0 c
            val r5'  = M.triml 0 c
            val r5'' = M.keepr 5 c

            val a = (M.triml ~1 c; false)
                      handle Subscript => true
            val b = (M.triml 6 c; false)
                      handle Subscript => true
            val c' = (M.trimr ~1 c; false)
                      handle Subscript => true
            val d = (M.trimr 6 c; false)
                      handle Subscript => true
            val e = (M.keepl ~1 c; false)
                      handle Subscript => true
            val f = (M.keepl 6 c; false)
                      handle Subscript => true
            val g = (M.keepr ~1 c; false)
                      handle Subscript => true
            val h = (M.keepr 6 c; false)
                      handle Subscript => true
            val i = (M.splitAt (c, ~1); false)
                      handle Subscript => true
            val j = (M.splitAt (c, 6); false)
                      handle Subscript => true
          in
            M.toList l1   = [chr 0, chr 1] andalso
            M.toList l1'  = [chr 0, chr 1] andalso
            M.toList l1'' = [chr 0, chr 1] andalso
            M.toList l2   = [chr 0, chr 1, chr 2, chr 3] andalso
            M.toList l2'  = [chr 0, chr 1, chr 2, chr 3] andalso
            M.toList l2'' = [chr 0, chr 1, chr 2, chr 3] andalso
            M.toList r1   = [chr 2, chr 3, chr 4] andalso
            M.toList r1'  = [chr 2, chr 3, chr 4] andalso
            M.toList r1'' = [chr 2, chr 3, chr 4] andalso
            M.toList r2   = [chr 4] andalso
            M.toList r2'  = [chr 4] andalso
            M.toList r2'' = [chr 4] andalso
            M.toList l3   = [chr 0] andalso
            M.toList l3'  = [chr 0] andalso
            M.toList l3'' = [chr 0] andalso
            M.toList l4   = [chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList l4'  = [chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList l4'' = [chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList l5   = [] andalso
            M.toList l5'  = [] andalso
            M.toList l5'' = [] andalso
            M.toList r3   = [chr 1, chr 2, chr 3, chr 4] andalso
            M.toList r3'  = [chr 1, chr 2, chr 3, chr 4] andalso
            M.toList r3'' = [chr 1, chr 2, chr 3, chr 4] andalso
            M.toList r4   = [] andalso
            M.toList r4'  = [] andalso
            M.toList r4'' = [] andalso
            M.toList r5   = [chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList r5'  = [chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList r5'' = [chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            a andalso b andalso c' andalso d andalso
            e andalso f andalso g  andalso h andalso
            i andalso j
          end
    
    fun testSplitl () =
          let
            val c = M.tabulate (5, fn x => chr x)
            val (l1, r1) = M.splitl (fn i => ord i <= 1) c
            val (l2, r2) = M.splitl (fn i => ord i <= 3) c
            
            val (l3, r3) = M.splitl (fn i => ord i <= 0) c
            val (l4, r4) = M.splitl (fn i => ord i <= 4) c
            val (l5, r5) = M.splitl (fn i => ord i <  0) c
            
            val l1'  = M.takel (fn i => ord i <= 1) c
            val l2'  = M.takel (fn i => ord i <= 3) c
            val r1'  = M.dropl (fn i => ord i <= 1) c
            val r2'  = M.dropl (fn i => ord i <= 3) c
            
            val l3'  = M.takel (fn i => ord i <= 0) c
            val l4'  = M.takel (fn i => ord i <= 4) c
            val l5'  = M.takel (fn i => ord i <  0) c
            val r3'  = M.dropl (fn i => ord i <= 0) c
            val r4'  = M.dropl (fn i => ord i <= 4) c
            val r5'  = M.dropl (fn i => ord i <  0) c

            (* These test-cases with repeated elements are important! *)
            val cc = M.append (c, c)
            
            val (ll1, rr1) = M.splitl (fn e => ord e <= 1) cc
            val (ll2, rr2) = M.splitl (fn e => ord e <= 3) cc
            
            val (ll3, rr3) = M.splitl (fn e => ord e <= 0) cc
            val (ll4, rr4) = M.splitl (fn e => ord e <= 4) cc
            val (ll5, rr5) = M.splitl (fn e => ord e <  0) cc

            val ll1'  = M.takel (fn e => ord e <= 1) cc
            val ll2'  = M.takel (fn e => ord e <= 3) cc
            val rr1'  = M.dropl (fn e => ord e <= 1) cc
            val rr2'  = M.dropl (fn e => ord e <= 3) cc
            
            val ll3'  = M.takel (fn e => ord e <= 0) cc
            val ll4'  = M.takel (fn e => ord e <= 4) cc
            val ll5'  = M.takel (fn e => ord e <  0) cc
            val rr3'  = M.dropl (fn e => ord e <= 0) cc
            val rr4'  = M.dropl (fn e => ord e <= 4) cc
            val rr5'  = M.dropl (fn e => ord e <  0) cc
          in
            M.toList l1   = [chr 0, chr 1] andalso
            M.toList l1'  = [chr 0, chr 1] andalso
            M.toList l2   = [chr 0, chr 1, chr 2, chr 3] andalso
            M.toList l2'  = [chr 0, chr 1, chr 2, chr 3] andalso
            M.toList r1   = [chr 2, chr 3, chr 4] andalso
            M.toList r1'  = [chr 2, chr 3, chr 4] andalso
            M.toList r2   = [chr 4] andalso
            M.toList r2'  = [chr 4] andalso
            M.toList l3   = [chr 0] andalso
            M.toList l3'  = [chr 0] andalso
            M.toList l4   = [chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList l4'  = [chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList l5   = [] andalso
            M.toList l5'  = [] andalso
            M.toList r3   = [chr 1, chr 2, chr 3, chr 4] andalso
            M.toList r3'  = [chr 1, chr 2, chr 3, chr 4] andalso
            M.toList r4   = [] andalso
            M.toList r4'  = [] andalso
            M.toList r5   = [chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList r5'  = [chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList ll1   = [chr 0, chr 1] andalso
            M.toList ll1'  = [chr 0, chr 1] andalso
            M.toList ll2   = [chr 0, chr 1, chr 2, chr 3] andalso
            M.toList ll2'  = [chr 0, chr 1, chr 2, chr 3] andalso
            M.toList rr1   = [chr 2, chr 3, chr 4, chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList rr1'  = [chr 2, chr 3, chr 4, chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList rr2   = [chr 4, chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList rr2'  = [chr 4, chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList ll3   = [chr 0] andalso
            M.toList ll3'  = [chr 0] andalso
            M.toList ll4   = [chr 0, chr 1, chr 2, chr 3, chr 4, chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList ll4'  = [chr 0, chr 1, chr 2, chr 3, chr 4, chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList ll5   = [] andalso
            M.toList ll5'  = [] andalso
            M.toList rr3   = [chr 1, chr 2, chr 3, chr 4, chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList rr3'  = [chr 1, chr 2, chr 3, chr 4, chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList rr4   = [] andalso
            M.toList rr4'  = [] andalso
            M.toList rr5   = [chr 0, chr 1, chr 2, chr 3, chr 4, chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList rr5'  = [chr 0, chr 1, chr 2, chr 3, chr 4, chr 0, chr 1, chr 2, chr 3, chr 4]
          end
    
    fun testSplitr () =
          let
            val c = M.tabulate (5, fn x => chr x)
            val (l1, r1) = M.splitr (fn i => ord i > 1) c
            val (l2, r2) = M.splitr (fn i => ord i > 3) c
            
            val (l3, r3) = M.splitr (fn i => ord i > 0) c
            val (l4, r4) = M.splitr (fn i => ord i > 4) c
            val (l5, r5) = M.splitr (fn i => ord i >= 0) c

            val l1'  = M.dropr (fn i => ord i > 1) c
            val l2'  = M.dropr (fn i => ord i > 3) c
            val r1'  = M.taker (fn i => ord i > 1) c
            val r2'  = M.taker (fn i => ord i > 3) c
            
            val l3'  = M.dropr (fn i => ord i > 0) c
            val l4'  = M.dropr (fn i => ord i > 4) c
            val l5'  = M.dropr (fn i => ord i >= 0) c
            val r3'  = M.taker (fn i => ord i > 0) c
            val r4'  = M.taker (fn i => ord i > 4) c
            val r5'  = M.taker (fn i => ord i >= 0) c

            (* These test-cases with repeated elements are important! *)
            val cc = M.append (c, c)
            
            val (ll1, rr1) = M.splitr (fn e => ord e > 1) cc
            val (ll2, rr2) = M.splitr (fn e => ord e > 3) cc
            
            val (ll3, rr3) = M.splitr (fn e => ord e > 0) cc
            val (ll4, rr4) = M.splitr (fn e => ord e > 4) cc
            val (ll5, rr5) = M.splitr (fn e => ord e >= 0) cc

            val ll1'  = M.dropr (fn e => ord e > 1) cc
            val ll2'  = M.dropr (fn e => ord e > 3) cc
            val rr1'  = M.taker (fn e => ord e > 1) cc
            val rr2'  = M.taker (fn e => ord e > 3) cc
            
            val ll3'  = M.dropr (fn e => ord e > 0) cc
            val ll4'  = M.dropr (fn e => ord e > 4) cc
            val ll5'  = M.dropr (fn e => ord e >= 0) cc
            val rr3'  = M.taker (fn e => ord e > 0) cc
            val rr4'  = M.taker (fn e => ord e > 4) cc
            val rr5'  = M.taker (fn e => ord e >= 0) cc
          in
            M.toList l1   = [chr 0, chr 1] andalso
            M.toList l1'  = [chr 0, chr 1] andalso
            M.toList l2   = [chr 0, chr 1, chr 2, chr 3] andalso
            M.toList l2'  = [chr 0, chr 1, chr 2, chr 3] andalso
            M.toList r1   = [chr 2, chr 3, chr 4] andalso
            M.toList r1'  = [chr 2, chr 3, chr 4] andalso
            M.toList r2   = [chr 4] andalso
            M.toList r2'  = [chr 4] andalso
            M.toList l3   = [chr 0] andalso
            M.toList l3'  = [chr 0] andalso
            M.toList l4   = [chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList l4'  = [chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList l5   = [] andalso
            M.toList l5'  = [] andalso
            M.toList r3   = [chr 1, chr 2, chr 3, chr 4] andalso
            M.toList r3'  = [chr 1, chr 2, chr 3, chr 4] andalso
            M.toList r4   = [] andalso
            M.toList r4'  = [] andalso
            M.toList r5   = [chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList r5'  = [chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList ll1   = [chr 0, chr 1, chr 2, chr 3, chr 4, chr 0, chr 1] andalso
            M.toList ll1'  = [chr 0, chr 1, chr 2, chr 3, chr 4, chr 0, chr 1] andalso
            M.toList ll2   = [chr 0, chr 1, chr 2, chr 3, chr 4, chr 0, chr 1, chr 2, chr 3] andalso
            M.toList ll2'  = [chr 0, chr 1, chr 2, chr 3, chr 4, chr 0, chr 1, chr 2, chr 3] andalso
            M.toList rr1   = [chr 2, chr 3, chr 4] andalso
            M.toList rr1'  = [chr 2, chr 3, chr 4] andalso
            M.toList rr2   = [chr 4] andalso
            M.toList rr2'  = [chr 4] andalso
            M.toList ll3   = [chr 0, chr 1, chr 2, chr 3, chr 4, chr 0] andalso
            M.toList ll3'  = [chr 0, chr 1, chr 2, chr 3, chr 4, chr 0] andalso
            M.toList ll4   = [chr 0, chr 1, chr 2, chr 3, chr 4, chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList ll4'  = [chr 0, chr 1, chr 2, chr 3, chr 4, chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList ll5   = [] andalso
            M.toList ll5'  = [] andalso
            M.toList rr3   = [chr 1, chr 2, chr 3, chr 4] andalso
            M.toList rr3'  = [chr 1, chr 2, chr 3, chr 4] andalso
            M.toList rr4   = [] andalso
            M.toList rr4'  = [] andalso
            M.toList rr5   = [chr 0, chr 1, chr 2, chr 3, chr 4, chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList rr5'  = [chr 0, chr 1, chr 2, chr 3, chr 4, chr 0, chr 1, chr 2, chr 3, chr 4]
          end

    fun testSplitli () =
          let
            val c = M.tabulate (5, fn x => chr x)
            val (l1, r1) = M.splitli (fn (i, _) => i <= 1) c
            val (l2, r2) = M.splitli (fn (i, _) => i <= 3) c
            
            val (l3, r3) = M.splitli (fn (i, _) => i <= 0) c
            val (l4, r4) = M.splitli (fn (i, _) => i <= 4) c
            val (l5, r5) = M.splitli (fn (i, _) => i <  0) c

            val l1'  = M.takeli (fn (i, _) => i <= 1) c
            val l2'  = M.takeli (fn (i, _) => i <= 3) c
            val r1'  = M.dropli (fn (i, _) => i <= 1) c
            val r2'  = M.dropli (fn (i, _) => i <= 3) c
            
            val l3'  = M.takeli (fn (i, _) => i <= 0) c
            val l4'  = M.takeli (fn (i, _) => i <= 4) c
            val l5'  = M.takeli (fn (i, _) => i <  0) c
            val r3'  = M.dropli (fn (i, _) => i <= 0) c
            val r4'  = M.dropli (fn (i, _) => i <= 4) c
            val r5'  = M.dropli (fn (i, _) => i <  0) c

            (* We have tabulated with fn x => x, so indices should be
               identical for all functions. *)
            val (lIndexSplit, rIndexSplit) = M.splitli (fn (i, e) => chr i = e) c
            val indexTake = M.takeli (fn (i, e) => chr i = e) c
            val indexDrop = M.dropli (fn (i, e) => chr i = e) c

            (* These test-cases with repeated elements are important! *)
            val cc = M.append (c, c)
            
            val (ll1, rr1) = M.splitli (fn (_, e) => ord e <= 1) cc
            val (ll2, rr2) = M.splitli (fn (_, e) => ord e <= 3) cc
            
            val (ll3, rr3) = M.splitli (fn (_, e) => ord e <= 0) cc
            val (ll4, rr4) = M.splitli (fn (_, e) => ord e <= 4) cc
            val (ll5, rr5) = M.splitli (fn (_, e) => ord e <  0) cc

            val ll1'  = M.takeli (fn (_, e) => ord e <= 1) cc
            val ll2'  = M.takeli (fn (_, e) => ord e <= 3) cc
            val rr1'  = M.dropli (fn (_, e) => ord e <= 1) cc
            val rr2'  = M.dropli (fn (_, e) => ord e <= 3) cc
            
            val ll3'  = M.takeli (fn (_, e) => ord e <= 0) cc
            val ll4'  = M.takeli (fn (_, e) => ord e <= 4) cc
            val ll5'  = M.takeli (fn (_, e) => ord e <  0) cc
            val rr3'  = M.dropli (fn (_, e) => ord e <= 0) cc
            val rr4'  = M.dropli (fn (_, e) => ord e <= 4) cc
            val rr5'  = M.dropli (fn (_, e) => ord e <  0) cc
          in
            M.toList l1   = [chr 0, chr 1] andalso
            M.toList l1'  = [chr 0, chr 1] andalso
            M.toList l2   = [chr 0, chr 1, chr 2, chr 3] andalso
            M.toList l2'  = [chr 0, chr 1, chr 2, chr 3] andalso
            M.toList r1   = [chr 2, chr 3, chr 4] andalso
            M.toList r1'  = [chr 2, chr 3, chr 4] andalso
            M.toList r2   = [chr 4] andalso
            M.toList r2'  = [chr 4] andalso
            M.toList l3   = [chr 0] andalso
            M.toList l3'  = [chr 0] andalso
            M.toList l4   = [chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList l4'  = [chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList l5   = [] andalso
            M.toList l5'  = [] andalso
            M.toList r3   = [chr 1, chr 2, chr 3, chr 4] andalso
            M.toList r3'  = [chr 1, chr 2, chr 3, chr 4] andalso
            M.toList r4   = [] andalso
            M.toList r4'  = [] andalso
            M.toList r5   = [chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList r5'  = [chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList lIndexSplit = [chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList rIndexSplit = [] andalso
            M.toList indexTake = [chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList indexDrop = [] andalso
            M.toList ll1   = [chr 0, chr 1] andalso
            M.toList ll1'  = [chr 0, chr 1] andalso
            M.toList ll2   = [chr 0, chr 1, chr 2, chr 3] andalso
            M.toList ll2'  = [chr 0, chr 1, chr 2, chr 3] andalso
            M.toList rr1   = [chr 2, chr 3, chr 4, chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList rr1'  = [chr 2, chr 3, chr 4, chr 0, chr 1, chr 2, chr 3, chr 4] andalso    
            M.toList rr2   = [chr 4, chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList rr2'  = [chr 4, chr 0, chr 1, chr 2, chr 3, chr 4] andalso   
            M.toList ll3   = [chr 0] andalso
            M.toList ll3'  = [chr 0] andalso
            M.toList ll4   = [chr 0, chr 1, chr 2, chr 3, chr 4, chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList ll4'  = [chr 0, chr 1, chr 2, chr 3, chr 4, chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList ll5   = [] andalso
            M.toList ll5'  = [] andalso
            M.toList rr3   = [chr 1, chr 2, chr 3, chr 4, chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList rr3'  = [chr 1, chr 2, chr 3, chr 4, chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList rr4   = [] andalso
            M.toList rr4'  = [] andalso
            M.toList rr5   = [chr 0, chr 1, chr 2, chr 3, chr 4, chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList rr5'  = [chr 0, chr 1, chr 2, chr 3, chr 4, chr 0, chr 1, chr 2, chr 3, chr 4]
          end
    
    fun testSplitri () =
          let
            val c = M.tabulate (5, fn x => chr x)

            val (l1, r1) = M.splitri (fn (i, _) => i > 1) c
            val (l2, r2) = M.splitri (fn (i, _) => i > 3) c
            
            val (l3, r3) = M.splitri (fn (i, _) => i > 0) c
            val (l4, r4) = M.splitri (fn (i, _) => i > 4) c
            val (l5, r5) = M.splitri (fn (i, _) => i >= 0) c

            val l1'  = M.dropri (fn (i, _) => i > 1) c
            val l2'  = M.dropri (fn (i, _) => i > 3) c
            val r1'  = M.takeri (fn (i, _) => i > 1) c
            val r2'  = M.takeri (fn (i, _) => i > 3) c
            
            val l3'  = M.dropri (fn (i, _) => i > 0) c
            val l4'  = M.dropri (fn (i, _) => i > 4) c
            val l5'  = M.dropri (fn (i, _) => i >= 0) c
            val r3'  = M.takeri (fn (i, _) => i > 0) c
            val r4'  = M.takeri (fn (i, _) => i > 4) c
            val r5'  = M.takeri (fn (i, _) => i >= 0) c

            (* We have tabulated with fn x => x, so indices should be
               identical for all functions. *)
            val (lIndexSplit, rIndexSplit) = M.splitri (fn (i, e) => chr i = e) c
            val indexTake = M.takeri (fn (i, e) => chr i = e) c
            val indexDrop = M.dropri (fn (i, e) => chr i = e) c

            (* These test-cases with repeated elements are important! *)
            val cc = M.append (c, c)
            
            val (ll1, rr1) = M.splitri (fn (_, e) => ord e > 1) cc
            val (ll2, rr2) = M.splitri (fn (_, e) => ord e > 3) cc
            
            val (ll3, rr3) = M.splitri (fn (_, e) => ord e > 0) cc
            val (ll4, rr4) = M.splitri (fn (_, e) => ord e > 4) cc
            val (ll5, rr5) = M.splitri (fn (_, e) => ord e >= 0) cc

            val ll1'  = M.dropri (fn (_, e) => ord e > 1) cc
            val ll2'  = M.dropri (fn (_, e) => ord e > 3) cc
            val rr1'  = M.takeri (fn (_, e) => ord e > 1) cc
            val rr2'  = M.takeri (fn (_, e) => ord e > 3) cc
            
            val ll3'  = M.dropri (fn (_, e) => ord e > 0) cc
            val ll4'  = M.dropri (fn (_, e) => ord e > 4) cc
            val ll5'  = M.dropri (fn (_, e) => ord e >= 0) cc
            val rr3'  = M.takeri (fn (_, e) => ord e > 0) cc
            val rr4'  = M.takeri (fn (_, e) => ord e > 4) cc
            val rr5'  = M.takeri (fn (_, e) => ord e >= 0) cc
          in
            M.toList l1   = [chr 0, chr 1] andalso
            M.toList l1'  = [chr 0, chr 1] andalso
            M.toList l2   = [chr 0, chr 1, chr 2, chr 3] andalso
            M.toList l2'  = [chr 0, chr 1, chr 2, chr 3] andalso
            M.toList r1   = [chr 2, chr 3, chr 4] andalso
            M.toList r1'  = [chr 2, chr 3, chr 4] andalso
            M.toList r2   = [chr 4] andalso
            M.toList r2'  = [chr 4] andalso
            M.toList l3   = [chr 0] andalso
            M.toList l3'  = [chr 0] andalso
            M.toList l4   = [chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList l4'  = [chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList l5   = [] andalso
            M.toList l5'  = [] andalso
            M.toList r3   = [chr 1, chr 2, chr 3, chr 4] andalso
            M.toList r3'  = [chr 1, chr 2, chr 3, chr 4] andalso
            M.toList r4   = [] andalso
            M.toList r4'  = [] andalso
            M.toList r5   = [chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList r5'  = [chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList lIndexSplit = [] andalso
            M.toList rIndexSplit = [chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList indexTake = [chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList indexDrop = [] andalso
            M.toList ll1   = [chr 0, chr 1, chr 2, chr 3, chr 4, chr 0, chr 1] andalso
            M.toList ll1'  = [chr 0, chr 1, chr 2, chr 3, chr 4, chr 0, chr 1] andalso
            M.toList ll2   = [chr 0, chr 1, chr 2, chr 3, chr 4, chr 0, chr 1, chr 2, chr 3] andalso
            M.toList ll2'  = [chr 0, chr 1, chr 2, chr 3, chr 4, chr 0, chr 1, chr 2, chr 3] andalso
            M.toList rr1   = [chr 2, chr 3, chr 4] andalso
            M.toList rr1'  = [chr 2, chr 3, chr 4] andalso
            M.toList rr2   = [chr 4] andalso
            M.toList rr2'  = [chr 4] andalso
            M.toList ll3   = [chr 0, chr 1, chr 2, chr 3, chr 4, chr 0] andalso
            M.toList ll3'  = [chr 0, chr 1, chr 2, chr 3, chr 4, chr 0] andalso
            M.toList ll4   = [chr 0, chr 1, chr 2, chr 3, chr 4, chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList ll4'  = [chr 0, chr 1, chr 2, chr 3, chr 4, chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList ll5   = [] andalso
            M.toList ll5'  = [] andalso
            M.toList rr3   = [chr 1, chr 2, chr 3, chr 4] andalso
            M.toList rr3'  = [chr 1, chr 2, chr 3, chr 4] andalso
            M.toList rr4   = [] andalso
            M.toList rr4'  = [] andalso
            M.toList rr5   = [chr 0, chr 1, chr 2, chr 3, chr 4, chr 0, chr 1, chr 2, chr 3, chr 4] andalso
            M.toList rr5'  = [chr 0, chr 1, chr 2, chr 3, chr 4, chr 0, chr 1, chr 2, chr 3, chr 4]
          end
    
    fun test () =
          testSimple () andalso
          testBasic () andalso
          testBasici () andalso
          testTabulatelr () andalso
          testRevAppend () andalso
          testConcat () andalso
          testApp () andalso
          testAppi () andalso
          testMap () andalso
          testMapi () andalso
          testFold () andalso
          testFoldi () andalso
          testFoldsep () andalso
          testFoldsepi () andalso
          testListReps () andalso
          testFilter () andalso
          testFilteri () andalso
          testSplitAt () andalso
          testSplitl () andalso
          testSplitr () andalso
          testSplitli () andalso
          testSplitri ()
  end

structure TestAnoqString = FTestAnoqString(structure M = String)
